#!/usr/bin/perl

use strict;
use Sendmail::Milter;
use Getopt::Long;
use File::Basename;
use Sys::Syslog qw(:DEFAULT setlogsock);

my($version) = '0.04';
my($author) = 'Victor Ustugov aka corvax';
my($email) = 'victor@corvax.kiev.ua';
my($info) = "backup perl mail filter $version 2003,2004 by $author $email";

my($default_syslog_facility) = 'local4';
my($MDA) = '';

my(@sendmail_config_defaults) = (
    '/etc/sendmail.cf',
    '/etc/mail/sendmail.cf'
);

sub helo_callback {
    my $ctx = shift;
    my $helohost = shift;

    my($href) = $ctx->getpriv();
    $$href{'helo'} = $helohost;
    $ctx->setpriv($href);

    return SMFIS_CONTINUE;
}

sub mailfrom_callback {
    my $ctx = shift;
    my $mail_from = shift;

    my($href) = $ctx->getpriv();
    $$href{'mail from'} = $mail_from;
    $ctx->setpriv($href);

    return SMFIS_CONTINUE;
}

sub rcptto_callback {
    my $ctx = shift;
    my $rcpt_to = shift;

    my($href) = $ctx->getpriv();
    $$href{'rcpt to'} .= ", " if ($$href{'rcpt to'});
    $$href{'rcpt to'} .= $rcpt_to;
    $ctx->setpriv($href);

    return SMFIS_CONTINUE;
}

sub header_callback {
    my $ctx = shift;
    my $header_field = shift;
    my $header_value = shift;

    my($href) = $ctx->getpriv();
    $$href{'data'} .= "$header_field: $header_value\n";
    $$href{'Return-Path'} = $header_value if ($header_field =~ /Return-Path/i);
    $ctx->setpriv($href);

    return SMFIS_CONTINUE;
}

sub eoh_callback {
    my $ctx = shift;

    my($href) = $ctx->getpriv();
    $$href{'data'} .= "\n";
    $ctx->setpriv($href);

    return SMFIS_CONTINUE;
}

sub body_callback {
    my $ctx = shift;
    my $body_part = shift;

    my($href) = $ctx->getpriv();
    $$href{'data'} .= $body_part;
    $ctx->setpriv($href);

    return SMFIS_CONTINUE;
}

sub eom_callback {
    my $ctx = shift;
    my $href = $ctx->getpriv();
    $ctx->setpriv($href);

    my($mail_from) = $$href{'mail from'};
    $mail_from =~ s/\s*\<\s*(.*)\s*\>\s*/$1/;
    my($MDA_command_line) = sprintf($MDA, $mail_from);
    unless (open(MDA, "|$MDA_command_line")) {
	syslog('notice', "couldn't execute MDA\n");
    } else {
	my($s) = $ctx->getsymval('s');
	$s = $$href{'helo'} unless ($s);
	my($auth_authen) = $ctx->getsymval('{auth_authen}');
	my($auth_type) = $ctx->getsymval('{auth_type}');
	my($auth_ssf) = $ctx->getsymval('{auth_ssf}');
	my($tls_version) = $ctx->getsymval('{tls_version}');
	my($cipher) = $ctx->getsymval('{cipher}');
	my($cipher_bits) = $ctx->getsymval('{cipher_bits}');
	my($verify) = $ctx->getsymval('{verify}');
	my($sender_address) = $ctx->getsymval('_');
	my($j) = $ctx->getsymval('j');
	my($v) = $ctx->getsymval('v');
	my($Z) = $ctx->getsymval('Z');
	my($r) = $ctx->getsymval('r');
	my($i) = $ctx->getsymval('i');
	my($u) = $ctx->getsymval('u');
	$u = $$href{'rcpt to'} unless ($u);
	my($b) = $ctx->getsymval('b');

	unless ($b) {
	    my(@WeekDayShortName) = qw(San Mon Tue Wed Thu Fri Sat);
	    my(@MontShortName) = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
	    my($sec,$min,$hour,$mday,$mon,$year,$wday) = (localtime())[0..6];
	    $b = sprintf("%s, %d %s %04d %02d:%02d:%02d %s",
		$WeekDayShortName[$wday],
		$mday,
		$MontShortName[$mon],
		$year + 1900,
		$hour, $min, $sec, GetTimeZone()
	    );
	}

#	$ctx->addheader('Return-Path', "<$mail_from>") unless ($$href{'Return-Path'});
	print(MDA "Return-Path: <$mail_from>\n") unless ($$href{'Return-Path'});

	print(MDA "X-Authenticated: ".($auth_type ? "with $auth_type " : "")."from $auth_authen at $j\n") if ($auth_authen);

	print(MDA
"Received: ".($s ? "from $s " : "").($sender_address ? "(".(! $s ? "from ":"").$sender_address.")
\t" : "").($auth_type ? "(authenticated".($auth_ssf eq "" ? "" : " bits=$auth_ssf").")
\t" : "")."by $j ($v/$Z)".($r ? " with $r" : "")." id $i".($tls_version ? "
\t(version=$tls_version cipher=$cipher bits=$cipher_bits verify=$verify)" : "").($u ? "
\tfor $u; " : ";
\t")."$b
");
	print(MDA $$href{'data'});

	syslog('notice', "couldn't send copy of message to the MDA\n") unless (close(MDA));
    }

#    $$href{'helo'} = undef;
#    $$href{'mail from'} = undef;
#    $$href{'body'} = undef;
#    $ctx->setpriv(undef);

#    delete($$href{'helo'});
#    delete($$href{'mail from'});
#    delete($$href{'body'});
#    $ctx->setpriv($href);

    return SMFIS_CONTINUE;
}

sub abort_callback {
    my $ctx = shift;
    $ctx->setpriv(undef);
    return SMFIS_CONTINUE;
}

sub close_callback {
    my $ctx = shift;
    $ctx->setpriv(undef);
    return SMFIS_CONTINUE;
}

my %my_callbacks = (
#	'connect' =>	\&connect_callback,
    'helo'	=> \&helo_callback,
    'envfrom'	=> \&mailfrom_callback,
    'envrcpt'	=> \&rcptto_callback,
    'header'	=> \&header_callback,
    'eoh'	=> \&eoh_callback,
    'body'	=> \&body_callback,
    'eom'	=> \&eom_callback,
    'abort'	=> \&abort_callback,
    'close'	=> \&close_callback,
);

BEGIN: {
    my($milter_name);
    my($sendmail_config);
    my($syslog_facility);
    my($help);
    my($version);

    #    
    GetOptions (
        'help'			=> \$help,
        'version'		=> \$version,
        'milter-name=s'		=> \$milter_name,
        'sendmail-config=s'	=> \$sendmail_config,
        'syslog-facility=s'	=> \$syslog_facility,
        'MDA=s'			=> \$MDA
    );

    $syslog_facility = $default_syslog_facility unless ($syslog_facility);

    my($script_name) = basename($0);

    if ($help) {
        print(
"$info

Usage: perl $script_name [OPTIONS]

General options:
  --milter-name        name of mail filter in sendmail.cf
  --sendmail-config    path to the sendmail.cf
  --MDA                command line for MDA

Additional options:
  --syslog-facility    syslog facility

Miscellaneous:
  --version            display the version of $script_name and exit
  --help               display this help and exit
");
        exit();
    }
    if ($version) {
        print("$info\n");
        exit();
    }

    #    sendmail',        
    $sendmail_config = find_sendmail_config(@sendmail_config_defaults) unless ($sendmail_config);

    #     ,       
    unless ($milter_name and $sendmail_config and $MDA) {
	print("name of mail filter in sendmail.cf is missing") unless ($milter_name);
        print("path to the sendmail.cf is missing") unless ($sendmail_config);
        print("command line for MDA is missing") unless ($MDA);
        print("

Usage: perl $script_name [OPTIONS]

Try `$script_name --help' for more options.

");
        exit();
    }

    setlogsock('unix');
    openlog($milter_name, 'cons,pid', $syslog_facility);

    my($conn) = Sendmail::Milter::auto_getconn($milter_name, $sendmail_config);

    print_msg("Found connection info for '$milter_name': $conn");

    if ($conn =~ /^local:(.+)$/) {
	my $unix_socket = $1;
	if (-e $unix_socket) {
	    print_msg("Attempting to unlink UNIX socket '$conn'...");
	    if (unlink($unix_socket) == 0) {
		print_msg("failed");
		exit();
	    }
	    print_msg("successful");
	}
    }

    if (not Sendmail::Milter::auto_setconn($milter_name, $sendmail_config)) {
	print_msg("Failed to detect connection information for '$milter_name'");
    } elsif (not Sendmail::Milter::register($milter_name, \%my_callbacks, SMFI_CURR_ACTS)) {
	print_msg("Failed to register callbacks for '$milter_name'");
    } else {
	print_msg("'$milter_name': Starting Sendmail::Milter $Sendmail::Milter::VERSION engine");
	if (Sendmail::Milter::main()) {
	    print_msg("'$milter_name': Successful exit from the Sendmail::Milter engine");
	} else {
	    print_msg("'$milter_name': Unsuccessful exit from the Sendmail::Milter engine");
	}
    }
    closelog();
}

sub print_msg() {
    my($msg) = @_;
    print($msg."\n");
    syslog('notice', $msg);
}

sub find_sendmail_config {
    my(@sendmail_config_defaults) = @_;
    my($file);
    foreach $file (@sendmail_config_defaults) {
        if (-f $file) {
            return($file);
        }
    }
    return('');
}

sub GetTimeZone {
	my($local, $gm, $tz, $prefix);
	$local = localtime();
	$gm    = gmtime();
	$tz    = ($local =~ /\s*\S*\s+\S*\s+\d+\s+(\d+):\d+:\d+\s+\d{4}/)[0] -
		($gm =~ /\s*\S*\s+\S*\s+\d+\s+(\d+):\d+:\d+\s+\d{4}/)[0];
	$tz += 24 if ($tz < -12);
	$tz -= 24 if ($tz > 12);
	if ($tz < 0) {
		$prefix = '-';
		$tz = -$tz;
	} else {
		$prefix = '+';
	}
	return(sprintf("$prefix%02d00", $tz));
}
